#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include "umatrix.h"
#include "fatals.h"

int N;
hrtime_t StartTime;
hrtime_t EndTime;

void printMatrix(int ** matrix, int nCol, int nRow);
int ** readMatrix(char * fname);

int main(int argc, char * argv[]) {
  int **A,**B,**C,**R;
  int row, col;

  if(argc!=5) {
    printf("Usage:\n %s N infile1 infile2 infile3\n",argv[0]);
    return 0;
  }

  N = atoi(argv[1]);

  A = readMatrix(argv[2]);
  if(A==NULL) fatal("Unable to read matrix from file: %s\n",argv[1]);

  B = readMatrix(argv[3]);
  if(B==NULL) fatal("Unable to read matrix from file: %s\n",argv[2]);

  C = readMatrix(argv[4]);
  if(C==NULL) fatal("Unable to read matrix from file: %s\n",argv[3]);

  // allocate space for matrix R
  R = (int **) malloc(N*sizeof(int*));
  if(R==NULL) fatal("Unable to allocate space for result matrix.",NULL);
  for(row=0;row<N;row++) {
    R[row] = (int*) malloc(N*sizeof(int));
    if(R[row]==NULL) fatal("Unable to allocate space for result matrix.",NULL);
  }

  StartTime = gethrtime();
  unimult(A,B,C,R,N);
  EndTime = gethrtime();

  printMatrix(A,N,N);

  // print elapsed time
  printf("Output in file: matrix\n");
  printf("Elapsed time:\t\t%lld ns\n",EndTime-StartTime);

  return 0;
}

int ** readMatrix(char * fname) {
  char buff[16384];
  FILE * infile;
  char * token;
  int ** matrix;
  int row,col;

  infile = fopen(fname,"r");
  if(infile==NULL) return NULL;

  // allocate the matrix
  matrix = (int **) malloc(N*sizeof(int*));
  if(matrix==NULL)
    fatal("Unable to allocate memory for matrix.",NULL);
  for(row=0;row<N;row++) {
    matrix[row] = (int *) malloc(N*sizeof(int));
    if(matrix[row]==NULL)
      fatal("Unable to allocate memory for matrix.",NULL);
  }

  // now loop and read in the matrix
  row = 0;
  col = 0;
  while(fgets(buff,16383,infile) ) {
    token = strtok(buff," \t\n");
    while(col<N) {
      if(token==NULL) {
        // N was improperly specified
        fatal("Specified N not compatible with input file.\n",NULL);
      }
      matrix[row][col] = atoi(token);
      col++;
      token = strtok(NULL," \t\n");
    }
    row++;
    col = 0;
    if(row>=N) break;
  }

  fclose(infile);

  return matrix;

}

void printMatrix(int ** matrix, int nCol, int nRow) {
  int col, row;
  FILE * outfile;

  outfile = fopen("matrix","w");
  if(outfile==NULL) return;

  for(row=0;row<nRow;row++) {
    for(col=0;col<nCol;col++) {
      fprintf(outfile,"%7.1i",matrix[row][col]);
    }
    fprintf(outfile,"\n");
  }

  fclose(outfile);
}

